@@ -1,14 +1,14 @@
-## ---- message=FALSE, warning=FALSE,echo=FALSE----------------------------
-library(knitcitations)
+## ---- message=FALSE, warning=FALSE,echo=FALSE---------------------------------
+# library(knitcitations)
 # cleanbib()
 # options("citation_format" = "pandoc")
 # r<-citep("10.1101/729285")
 # write.bibtex(file="references.bib")
 
-## ----eval=FALSE----------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #  install.packages("coxmeg", repos="http://R-Forge.R-project.org")
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 library(coxmeg)
 library(MASS)
 library(Matrix)
@@ -25,7 +25,7 @@ sigma <- as.matrix(bdiag(mat_list))
 sigma = as(sigma,'dgCMatrix')
 
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 n = nrow(sigma)
 tau_var <- 0.2
 x <- mvrnorm(1, rep(0,n), tau_var*sigma)
@@ -39,11 +39,11 @@ head(outcome)
 sigma[1:5,1:5]
 
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 re = coxmeg(outcome,sigma,type='bd',X=pred,order=1,detap='diagonal')
 re
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 library(coxme)
 bls <- c(1)
 for(i in (size[1]-1):1)
@@ -52,7 +52,7 @@ tmat <- bdsmatrix(blocksize=size, blocks=rep(bls,n_f),dimnames=list(as.character
 re_coxme = coxme(Surv(outcome[,1],outcome[,2])~as.matrix(pred)+(1|as.character(1:n)), varlist=list(tmat),ties='breslow')
 re_coxme
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 library(coxmeg)
 bed = system.file("extdata", "example_null.bed", package = "coxmeg")
 bed = substr(bed,1,nchar(bed)-4)
@@ -74,25 +74,25 @@ sigma <- as.matrix(bdiag(mat_list))
 re = coxmeg_plink(pheno,sigma,type='bd',bed=bed,tmp_dir=tempdir(),cov_file=cov,verbose=FALSE)
 re
 
-## ----eval=FALSE----------------------------------------------------------
+## ----eval=FALSE---------------------------------------------------------------
 #  re = coxmeg_plink(pheno,sigma,type='bd',cov_file=cov,verbose=FALSE)
 #  re = coxmeg_plink(pheno,sigma,type='bd',bed=bed,tmp_dir=tempdir(),tau=re$tau,cov_file=cov,verbose=FALSE)
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 geno = matrix(rbinom(nrow(sigma)*10,2,runif(nrow(sigma)*10,0.05,0.5)),nrow(sigma),10)
 pheno_m = read.table(pheno)
 re = coxmeg_m(geno,pheno_m[,3:4],sigma,type='bd',verbose=FALSE)
 re
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 re = coxmeg_plink(pheno,sigma,type='dense',bed=bed,tmp_dir=tempdir(),cov_file=cov,detap='slq',verbose=FALSE,solver=2)
 re
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 re = coxmeg_plink(pheno,sigma,type='dense',bed=bed,tmp_dir=tempdir(),tau=re$tau,cov_file=cov,detap='slq',verbose=FALSE,solver=2,score=TRUE)
 re
 
-## ----echo=TRUE-----------------------------------------------------------
+## ----echo=TRUE----------------------------------------------------------------
 sigma[2,1] = sigma[1,2] = 1
 re = coxmeg_plink(pheno,sigma,type='bd',cov_file=cov,verbose=FALSE,spd=FALSE)
 re
